import { RequestHandler } from "express";
import { config } from "@/src/services/configService";
import allShipFeatures from "@/static/fixed_responses/allShipFeatures.json";
import { getAccountIdForRequest } from "@/src/services/loginService";
import { getPersonalRooms } from "@/src/services/personalRoomsService";
import { getShip } from "@/src/services/shipService";
import { toOid } from "@/src/helpers/inventoryHelpers";
import { IGetShipResponse } from "@/src/types/shipTypes";
import { IPersonalRoomsClient } from "@/src/types/personalRoomsTypes";
import { getLoadout } from "@/src/services/loadoutService";
import { getRandomElement } from "@/src/services/rngService";

export const getShipController: RequestHandler = async (req, res) => {
    const accountId = await getAccountIdForRequest(req);
    const personalRoomsDb = await getPersonalRooms(accountId);

    // Setup gardening if it's missing. Maybe should be done as part of some quest completion in the future.
    if (personalRoomsDb.Apartment.Gardening.Planters.length == 0) {
        const plantTypes = [
            "/Lotus/Types/Items/Plants/MiscItems/DuvxDuviriGrowingPlantA",
            "/Lotus/Types/Items/Plants/MiscItems/DuvxDuviriGrowingPlantB",
            "/Lotus/Types/Items/Plants/MiscItems/DuvxDuviriGrowingPlantC",
            "/Lotus/Types/Items/Plants/MiscItems/DuvxDuviriGrowingPlantD",
            "/Lotus/Types/Items/Plants/MiscItems/DuvxDuviriGrowingPlantE",
            "/Lotus/Types/Items/Plants/MiscItems/DuvxDuviriGrowingPlantF"
        ];
        const endTime = new Date(Date.now() + 79200_000); // Plants will take 22 hours to grow
        personalRoomsDb.Apartment.Gardening = {
            Planters: [
                {
                    Name: "Garden0",
                    Plants: [
                        {
                            PlantType: getRandomElement(plantTypes),
                            EndTime: endTime,
                            PlotIndex: 0
                        },
                        {
                            PlantType: getRandomElement(plantTypes),
                            EndTime: endTime,
                            PlotIndex: 1
                        }
                    ]
                },
                {
                    Name: "Garden1",
                    Plants: [
                        {
                            PlantType: getRandomElement(plantTypes),
                            EndTime: endTime,
                            PlotIndex: 0
                        },
                        {
                            PlantType: getRandomElement(plantTypes),
                            EndTime: endTime,
                            PlotIndex: 1
                        }
                    ]
                },
                {
                    Name: "Garden2",
                    Plants: [
                        {
                            PlantType: getRandomElement(plantTypes),
                            EndTime: endTime,
                            PlotIndex: 0
                        },
                        {
                            PlantType: getRandomElement(plantTypes),
                            EndTime: endTime,
                            PlotIndex: 1
                        }
                    ]
                }
            ]
        };
        await personalRoomsDb.save();
    }

    const personalRooms = personalRoomsDb.toJSON<IPersonalRoomsClient>();
    const loadout = await getLoadout(accountId);
    const ship = await getShip(personalRoomsDb.activeShipId, "ShipAttachments SkinFlavourItem");

    const getShipResponse: IGetShipResponse = {
        ShipOwnerId: accountId,
        LoadOutInventory: { LoadOutPresets: loadout.toJSON() },
        Ship: {
            ...personalRooms.Ship,
            ShipId: toOid(personalRoomsDb.activeShipId),
            ShipInterior: {
                Colors: personalRooms.ShipInteriorColors,
                ShipAttachments: ship.ShipAttachments,
                SkinFlavourItem: ship.SkinFlavourItem
            },
            FavouriteLoadoutId: personalRooms.Ship.FavouriteLoadoutId
                ? toOid(personalRooms.Ship.FavouriteLoadoutId)
                : undefined
        },
        Apartment: personalRooms.Apartment,
        TailorShop: personalRooms.TailorShop
    };

    if (config.unlockAllShipFeatures) {
        getShipResponse.Ship.Features = allShipFeatures;
    }

    res.json(getShipResponse);
};
